@@ -24,13 +24,55 @@
 #define MAX_LINE_LENGTH BUFSIZ
 
 int
-imap_select_mechanism (char **mechlist)
+imap_greeting (void)
 {
   char *in;
 
   if (!readln (&in))
     return 0;
 
+  return 1;
+}
+
+int
+imap_has_starttls (void)
+{
+  char *in, *capa;
+  int has_tls = 0;
+
+  if (!writeln (". CAPABILITY"))
+    return 0;
+
+  if (!readln (&in))
+    return 0;
+
+  has_tls = strstr (in, "STARTTLS") != NULL;
+
+  if (!readln (&in))
+    return 0;
+
+  return has_tls;
+}
+
+int
+imap_starttls (void)
+{
+  char *in;
+
+  if (!writeln (". STARTTLS"))
+    return 0;
+
+  if (!readln (&in))
+    return 0;
+
+  return 1;
+}
+
+int
+imap_select_mechanism (char **mechlist)
+{
+  char *in;
+
   if (args_info.server_flag)
     {
       if (!args_info.quiet_given)
@@ -39,7 +81,7 @@ imap_select_mechanism (char **mechlist)
 	return 0;
       *mechlist = in;
     }
-  else				/* if (args_info.client_flag) */
+  else
     {
       if (!writeln (". CAPABILITY"))
 	return 0;
@@ -61,7 +103,13 @@ imap_select_mechanism (char **mechlist)
 int
 imap_authenticate (const char *mech)
 {
-  if (args_info.client_flag)
+  if (args_info.server_flag)
+    {
+      if (!args_info.quiet_given)
+	fprintf (stderr, _("Using mechanism:\n"));
+      puts (mech);
+    }
+  else
     {
       char input[MAX_LINE_LENGTH];
 
@@ -69,12 +117,6 @@ imap_authenticate (const char *mech)
       if (!writeln (input))
 	return 0;
     }
-  else
-    {
-      if (!args_info.quiet_given)
-	fprintf (stderr, _("Using mechanism:\n"));
-      puts (mech);
-    }
 
   return 1;
 }
@@ -84,10 +126,10 @@ imap_step_send (const char *data)
 {
   char input[MAX_LINE_LENGTH];
 
-  if (args_info.client_flag)
-    sprintf (input, "%s", data);
-  else
+  if (args_info.server_flag)
     sprintf (input, "+ %s", data);
+  else
+    sprintf (input, "%s", data);
   if (!writeln (input))
     return 0;
 
